@@ -3,6 +3,7 @@ from __future__ import division
 from __future__ import print_function
 
 import sys
+sys.path.append("../../..")
 import math
 import os
 import argparse
@@ -14,8 +15,7 @@ import torchvision
 import torchvision.transforms as transforms
 from torch.autograd import Variable
 from torch.nn import init
-from models import nin_gc
-from models import nin
+from models import nin_gc, nin
 
 import quantize
 
@@ -40,9 +40,15 @@ def save_state(model, best_acc):
             state['state_dict'][key.replace('module.', '')] = \
                     state['state_dict'].pop(key)
     if args.model_type == 0:
-        torch.save(state, 'models_save/nin.pth')
+        if args.prune_refine:
+            torch.save({'cfg': cfg, 'best_acc': best_acc, 'state_dict': state['state_dict']}, 'models_save/nin.pth')
+        else:
+            torch.save(state, 'models_save/nin.pth')
     else:
-        torch.save(state, 'models_save/nin_gc.pth')
+        if args.prune_refine:
+            torch.save({'cfg': cfg, 'best_acc': best_acc, 'state_dict': state['state_dict']}, 'models_save/nin_gc.pth')
+        else:
+            torch.save(state, 'models_save/nin_gc.pth')
 
 # 训练lr调整
 def adjust_learning_rate(optimizer, epoch):
@@ -194,15 +200,13 @@ if __name__=='__main__':
         print('******Prune Refine model******')
         #checkpoint = torch.load('../prune/models_save/nin_refine.pth')
         checkpoint = torch.load(args.prune_refine)
+        cfg = checkpoint['cfg']
         if args.model_type == 0:
             model = nin.Net(cfg=checkpoint['cfg'])
         else:
             model = nin_gc.Net(cfg=checkpoint['cfg'])
         model.load_state_dict(checkpoint['state_dict'])
         best_acc = 0
-        print('***ori_model***\n', model)
-        quantize.prepare(model, inplace=True, A=args.A, W=args.W)
-        print('\n***quant_model***\n', model)
     elif args.refine:
         print('******Float Refine model******')
         #checkpoint = torch.load('models_save/nin.pth')
@@ -213,9 +217,6 @@ if __name__=='__main__':
             model = nin_gc.Net()
         model.load_state_dict(state_dict)
         best_acc = 0
-        print('***ori_model***\n', model)
-        quantize.prepare(model, inplace=True, A=args.A, W=args.W)
-        print('\n***quant_model***\n', model)
     elif args.resume:
         print('******Reume model******')
         #checkpoint = torch.load('models_save/nin.pth')
@@ -226,9 +227,6 @@ if __name__=='__main__':
             model = nin_gc.Net()
         model.load_state_dict(checkpoint['state_dict'])
         best_acc = checkpoint['best_acc']
-        print('***ori_model***\n', model)
-        quantize.prepare(model, inplace=True, A=args.A, W=args.W)
-        print('\n***quant_model***\n', model)
     else:
         print('******Initializing model******')
         if args.model_type == 0:
@@ -245,9 +243,9 @@ if __name__=='__main__':
                 init.normal_(m.weight, 0, 0.01)
                 if m.bias is not None:
                     init.zeros_(m.bias)
-        print('***ori_model***\n', model)
-        quantize.prepare(model, inplace=True, A=args.A, W=args.W)
-        print('\n***quant_model***\n', model)
+    print('***ori_model***\n', model)
+    quantize.prepare(model, inplace=True, A=args.A, W=args.W)
+    print('\n***quant_model***\n', model)
 
     # cpu、gpu
     if not args.cpu:
