@@ -3,6 +3,7 @@ from __future__ import division
 from __future__ import print_function
 
 import sys
+sys.path.append("../../../..")
 import math
 import os
 import argparse
@@ -14,8 +15,7 @@ import torchvision
 import torchvision.transforms as transforms
 from torch.autograd import Variable
 from torch.nn import init
-from models import nin_gc
-from models import nin
+from models import nin_gc, nin
 
 import quantize
 
@@ -38,7 +38,10 @@ def save_state(model, best_acc):
             state['state_dict'][key.replace('module.', '')] = \
                     state['state_dict'].pop(key)
     if args.model_type == 0:
-        torch.save(state, 'models_save/nin.pth')
+        if args.bn_fuse == 1:
+            torch.save(state, 'models_save/nin_bn_fused.pth')
+        else:
+            torch.save(state, 'models_save/nin.pth')
     else:
         if args.bn_fuse == 1:
             torch.save(state, 'models_save/nin_gc_bn_fused.pth')
