@@ -108,12 +108,11 @@ class Round(Function):
         return grad_input
 
 class Quantizer(nn.Module):
-    def __init__(self, bits, observer, activation_weight_flag, device):
+    def __init__(self, bits, observer, activation_weight_flag):
         super(Quantizer, self).__init__()
         self.bits = bits
         self.observer = observer
         self.activation_weight_flag = activation_weight_flag
-        self.device = device
         #scale/zero_point/eps
         if self.observer.q_level == 'L':            
             self.register_buffer('scale', torch.ones((1), dtype=torch.float32))
@@ -124,9 +123,9 @@ class Quantizer(nn.Module):
         elif self.observer.q_level == 'FC':         
             self.register_buffer('scale', torch.ones((self.observer.out_channels, 1), dtype=torch.float32))
             self.register_buffer('zero_point', torch.zeros((self.observer.out_channels, 1), dtype=torch.float32))
-        self.register_buffer('eps', torch.tensor([torch.finfo(torch.float32).eps])) # eps(1.1921e-07)
+        self.eps = torch.tensor((torch.finfo(torch.float32).eps), dtype=torch.float32, device=self.observer.device) # eps(1.1921e-07)
 
-    def update_params(self):
+    def update_qparams(self):
         raise NotImplementedError
 
     # 取整(ste)
@@ -141,8 +140,9 @@ class Quantizer(nn.Module):
             print('！Binary quantization is not supported ！')
             assert self.bits != 1
         else:
-            self.observer(input)
-            self.update_params()  # update scale and zero_point
+            if self.training:
+                self.observer(input)
+                self.update_qparams()  # update scale and zero_point
             # 量化/反量化
             output = (torch.clamp(self.round(input / self.scale - self.zero_point), self.quant_min_val, self.quant_max_val) + self.zero_point) * self.scale
         return output
@@ -150,18 +150,18 @@ class Quantizer(nn.Module):
 class SignedQuantizer(Quantizer):
     def __init__(self, *args, **kwargs):
         super(SignedQuantizer, self).__init__(*args, **kwargs)
-        self.quant_min_val = torch.tensor((-(1 << (self.bits - 1))), device=self.device)
-        self.quant_max_val = torch.tensor(((1 << (self.bits - 1)) - 1), device=self.device)
+        self.quant_min_val = torch.tensor((-(1 << (self.bits - 1))), device=self.observer.device)
+        self.quant_max_val = torch.tensor(((1 << (self.bits - 1)) - 1), device=self.observer.device)
 
 class UnsignedQuantizer(Quantizer):
     def __init__(self, *args, **kwargs):
         super(UnsignedQuantizer, self).__init__(*args, **kwargs)
-        self.quant_min_val = torch.tensor((0), device=self.device)
-        self.quant_max_val = torch.tensor(((1 << self.bits) - 1), device=self.device)
+        self.quant_min_val = torch.tensor((0), device=self.observer.device)
+        self.quant_max_val = torch.tensor(((1 << self.bits) - 1), device=self.observer.device)
 
 # 对称量化
 class SymmetricQuantizer(SignedQuantizer):
-    def update_params(self):
+    def update_qparams(self):
         # quantized_range
         if self.activation_weight_flag == 0: 
             quant_range = float(torch.min(torch.abs(self.quant_min_val), torch.abs(self.quant_max_val)))    # weight               
@@ -174,7 +174,7 @@ class SymmetricQuantizer(SignedQuantizer):
 
 # 非对称量化
 class AsymmetricQuantizer(UnsignedQuantizer):
-    def update_params(self):
+    def update_qparams(self):
         quant_range = float(self.quant_max_val - self.quant_min_val)           # quantized_range
         float_range = self.observer.max_val - self.observer.min_val            # float_range
         self.scale = float_range / quant_range                                 # scale
@@ -199,39 +199,44 @@ class QuantConv2d(nn.Conv2d):
                  q_type=0,
                  q_level=0,
                  device='cpu',
-                 weight_observer=0):
+                 weight_observer=0,
+                 quant_inference=False):
         super(QuantConv2d, self).__init__(in_channels, out_channels, kernel_size, stride, padding, dilation, groups,
                                           bias, padding_mode)
+        self.quant_inference = quant_inference
         # 实例化量化器（A-layer级，W-channel级）
         if q_type == 0:
-            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1, device=device)
+            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1)
             if weight_observer == 0:
                 if q_level == 0:
-                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='C', out_channels=out_channels, device=device), activation_weight_flag=0, device=device)
+                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='C', out_channels=out_channels, device=device), activation_weight_flag=0)
                 else:
-                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=0, device=device)
+                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=0)
             else:
                 if q_level == 0:
-                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='C', out_channels=out_channels, device=device), activation_weight_flag=0, device=device)
+                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='C', out_channels=out_channels, device=device), activation_weight_flag=0)
                 else:
-                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=0, device=device)
+                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=0)
         else:
-            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
             if weight_observer == 0:
                 if q_level == 0:
-                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='C', out_channels=out_channels, device=device), activation_weight_flag=2, device=device)
+                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='C', out_channels=out_channels, device=device), activation_weight_flag=2)
                 else:
-                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
             else:
                 if q_level == 0:
-                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='C', out_channels=out_channels, device=device), activation_weight_flag=2, device=device)
+                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='C', out_channels=out_channels, device=device), activation_weight_flag=2)
                 else:
-                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
 
     def forward(self, input):
         # 量化A和W
         quant_input = self.activation_quantizer(input)
-        quant_weight = self.weight_quantizer(self.weight) 
+        if not self.quant_inference:
+            quant_weight = self.weight_quantizer(self.weight) 
+        else:
+            quant_weight = self.weight
         # 量化卷积
         output = F.conv2d(quant_input, quant_weight, self.bias, self.stride, self.padding, self.dilation,
                           self.groups)
@@ -253,25 +258,30 @@ class QuantConvTranspose2d(nn.ConvTranspose2d):
                  w_bits=8,
                  q_type=0,
                  device='cpu',
-                 weight_observer=0):
+                 weight_observer=0,
+                 quant_inference=False):
         super(QuantConvTranspose2d, self).__init__(in_channels, out_channels, kernel_size, stride, padding, output_padding, 
                                                    dilation, groups, bias, padding_mode)
+        self.quant_inference = quant_inference
         if q_type == 0:
-            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1, device=device)
+            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1)
             if weight_observer == 0:
-                self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=0, device=device)
+                self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=0)
             else:
-                self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=0, device=device)
+                self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=0)
         else:
-            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
             if weight_observer == 0:
-                self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+                self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
             else:
-                self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+                self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
 
     def forward(self, input):
         quant_input = self.activation_quantizer(input)
-        quant_weight = self.weight_quantizer(self.weight)
+        if not self.quant_inference:
+            quant_weight = self.weight_quantizer(self.weight)
+        else:
+            quant_weight = self.weight
         output = F.conv_transpose2d(quant_input, quant_weight, self.bias, self.stride, self.padding, self.output_padding,
                                     self.groups, self.dilation)
         return output
@@ -316,29 +326,29 @@ class QuantBNFuseConv2d(QuantConv2d):
         
         # 实例化量化器（A-layer级，W-channel级）
         if q_type == 0:
-            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1, device=device)
+            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1)
             if weight_observer == 0:
                 if q_level == 0:
-                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='C', out_channels=out_channels, device=device), activation_weight_flag=0, device=device)
+                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='C', out_channels=out_channels, device=device), activation_weight_flag=0)
                 else:
-                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=0, device=device)
+                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=0)
             else:
                 if q_level == 0:
-                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='C', out_channels=out_channels, device=device), activation_weight_flag=0, device=device)
+                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='C', out_channels=out_channels, device=device), activation_weight_flag=0)
                 else:
-                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=0, device=device)
+                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=0)
         else:
-            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
             if weight_observer == 0:
                 if q_level == 0:
-                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='C', out_channels=out_channels, device=device), activation_weight_flag=2, device=device)
+                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='C', out_channels=out_channels, device=device), activation_weight_flag=2)
                 else:
-                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
             else:
                 if q_level == 0:
-                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='C', out_channels=out_channels, device=device), activation_weight_flag=2, device=device)
+                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='C', out_channels=out_channels, device=device), activation_weight_flag=2)
                 else:
-                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
        
     def forward(self, input):
         # 训练态
@@ -391,35 +401,39 @@ class QuantBNFuseConv2d(QuantConv2d):
         return output
 
 class QuantLinear(nn.Linear):
-    def __init__(self, in_features, out_features, bias=True, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu', weight_observer=0):
+    def __init__(self, in_features, out_features, bias=True, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu', weight_observer=0, quant_inference=False):
         super(QuantLinear, self).__init__(in_features, out_features, bias)
+        self.quant_inference = quant_inference
         if q_type == 0:
-            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1, device=device)
+            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1)
             if weight_observer == 0:
                 if q_level == 0:
-                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='FC', out_channels=out_features, device=device), activation_weight_flag=0, device=device)
+                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='FC', out_channels=out_features, device=device), activation_weight_flag=0)
                 else:
-                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=0, device=device)
+                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=0)
             else:
                 if q_level == 0:
-                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='FC', out_channels=out_features, device=device), activation_weight_flag=0, device=device)
+                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='FC', out_channels=out_features, device=device), activation_weight_flag=0)
                 else:
-                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=0, device=device)
+                    self.weight_quantizer = SymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=0)
         else:
-            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
             if weight_observer == 0:
                 if q_level == 0:
-                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='FC', out_channels=out_features, device=device), activation_weight_flag=2, device=device)
+                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='FC', out_channels=out_features, device=device), activation_weight_flag=2)
                 else:
-                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
             else:
                 if q_level == 0:
-                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='FC', out_channels=out_features, device=device), activation_weight_flag=2, device=device)
+                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='FC', out_channels=out_features, device=device), activation_weight_flag=2)
                 else:
-                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+                    self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
     def forward(self, input):
         quant_input = self.activation_quantizer(input)
-        quant_weight = self.weight_quantizer(self.weight)
+        if not self.quant_inference:
+            quant_weight = self.weight_quantizer(self.weight)
+        else:
+            quant_weight = self.weight
         output = F.linear(quant_input, quant_weight, self.bias)
         return output
         
@@ -427,9 +441,9 @@ class QuantReLU(nn.ReLU):
     def __init__(self, inplace=False, a_bits=8, q_type=0, device='cpu'):
         super(QuantReLU, self).__init__(inplace)
         if q_type == 0:
-            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1, device=device)
+            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1)
         else:
-            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
 
     def forward(self, input):
         quant_input = self.activation_quantizer(input)
@@ -440,9 +454,9 @@ class QuantSigmoid(nn.Sigmoid):
     def __init__(self, a_bits=8, q_type=0, device='cpu'):
         super(QuantSigmoid, self).__init__()
         if q_type == 0:
-            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1, device=device)
+            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1)
         else:
-            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
 
     def forward(self, input):
         quant_input = self.activation_quantizer(input)
@@ -455,9 +469,9 @@ class QuantMaxPool2d(nn.MaxPool2d):
         super(QuantMaxPool2d, self).__init__(kernel_size, stride, padding, dilation,
                                              return_indices, ceil_mode)
         if q_type == 0:
-            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1, device=device)
+            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1)
         else:
-            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
 
     def forward(self, input):
         quant_input = self.activation_quantizer(input)
@@ -470,9 +484,9 @@ class QuantAvgPool2d(nn.AvgPool2d):
         super(QuantAvgPool2d, self).__init__(kernel_size, stride, padding, ceil_mode,
                                              count_include_pad, divisor_override)
         if q_type == 0:
-            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1, device=device)
+            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1)
         else:
-            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
 
     def forward(self, input):
         quant_input = self.activation_quantizer(input)
@@ -483,16 +497,16 @@ class QuantAdaptiveAvgPool2d(nn.AdaptiveAvgPool2d):
     def __init__(self, output_size, a_bits=8, q_type=0, device='cpu'):
         super(QuantAdaptiveAvgPool2d, self).__init__(output_size)
         if q_type == 0:
-            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1, device=device)
+            self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=1)
         else:
-            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2, device=device)
+            self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(q_level='L', out_channels=None, device=device), activation_weight_flag=2)
 
     def forward(self, input):
         quant_input = self.activation_quantizer(input)
         output = F.adaptive_avg_pool2d(quant_input, self.output_size)
         return output
 
-def add_quant_op(module, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu', weight_observer=0, bn_fuse=0):
+def add_quant_op(module, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu', weight_observer=0, bn_fuse=0, quant_inference=False):
     for name, child in module.named_children():
         if isinstance(child, nn.Conv2d):
             if bn_fuse:
@@ -501,11 +515,11 @@ def add_quant_op(module, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu',
             else:
                 if child.bias is not None:
                     quant_conv = QuantConv2d(child.in_channels, child.out_channels,
-                                             child.kernel_size, stride=child.stride, padding=child.padding, dilation=child.dilation, groups=child.groups, bias=True, padding_mode=child.padding_mode, a_bits=a_bits, w_bits=w_bits, q_type=q_type, q_level=q_level, device=device, weight_observer=weight_observer)
+                                             child.kernel_size, stride=child.stride, padding=child.padding, dilation=child.dilation, groups=child.groups, bias=True, padding_mode=child.padding_mode, a_bits=a_bits, w_bits=w_bits, q_type=q_type, q_level=q_level, device=device, weight_observer=weight_observer, quant_inference=quant_inference)
                     quant_conv.bias.data = child.bias
                 else:
                     quant_conv = QuantConv2d(child.in_channels, child.out_channels,
-                                             child.kernel_size, stride=child.stride, padding=child.padding, dilation=child.dilation, groups=child.groups, bias=False, padding_mode=child.padding_mode, a_bits=a_bits, w_bits=w_bits, q_type=q_type, q_level=q_level, device=device, weight_observer=weight_observer)
+                                             child.kernel_size, stride=child.stride, padding=child.padding, dilation=child.dilation, groups=child.groups, bias=False, padding_mode=child.padding_mode, a_bits=a_bits, w_bits=w_bits, q_type=q_type, q_level=q_level, device=device, weight_observer=weight_observer, quant_inference=quant_inference)
                 quant_conv.weight.data = child.weight
                 module._modules[name] = quant_conv
         elif isinstance(child, nn.BatchNorm2d):
@@ -522,24 +536,25 @@ def add_quant_op(module, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu',
                 quant_bn_fuse_conv.beta.data = child.bias
                 quant_bn_fuse_conv.running_mean.copy_(child.running_mean)
                 quant_bn_fuse_conv.running_var.copy_(child.running_var)
+                quant_bn_fuse_conv.eps = child.eps
                 module._modules[conv_name_temp] = quant_bn_fuse_conv
                 module._modules[name] = nn.Identity()
         elif isinstance(child, nn.ConvTranspose2d):
             if child.bias is not None:
                 quant_conv_transpose = QuantConvTranspose2d(child.in_channels, child.out_channels,
-                                                            child.kernel_size, stride=child.stride, padding=child.padding, output_padding=child.output_padding, dilation=child.dilation, groups=child.groups, bias=True, padding_mode=child.padding_mode, a_bits=a_bits, w_bits=w_bits, q_type=q_type, device=device, weight_observer=weight_observer)
+                                                            child.kernel_size, stride=child.stride, padding=child.padding, output_padding=child.output_padding, dilation=child.dilation, groups=child.groups, bias=True, padding_mode=child.padding_mode, a_bits=a_bits, w_bits=w_bits, q_type=q_type, device=device, weight_observer=weight_observer, quant_inference=quant_inference)
                 quant_conv_transpose.bias.data = child.bias
             else:
                 quant_conv_transpose = QuantConvTranspose2d(child.in_channels, child.out_channels,
-                                                            child.kernel_size, stride=child.stride, padding=child.padding, output_padding=child.output_padding, dilation=child.dilation, groups=child.groups, bias=False, padding_mode=child.padding_mode, a_bits=a_bits, w_bits=w_bits, q_type=q_type, device=device, weight_observer=weight_observer)
+                                                            child.kernel_size, stride=child.stride, padding=child.padding, output_padding=child.output_padding, dilation=child.dilation, groups=child.groups, bias=False, padding_mode=child.padding_mode, a_bits=a_bits, w_bits=w_bits, q_type=q_type, device=device, weight_observer=weight_observer, quant_inference=quant_inference)
             quant_conv_transpose.weight.data = child.weight
             module._modules[name] = quant_conv_transpose
         elif isinstance(child, nn.Linear):
             if child.bias is not None:
-                quant_linear = QuantLinear(child.in_features, child.out_features, bias=True, a_bits=a_bits, w_bits=w_bits, q_type=q_type, q_level=q_level, device=device, weight_observer=weight_observer)
+                quant_linear = QuantLinear(child.in_features, child.out_features, bias=True, a_bits=a_bits, w_bits=w_bits, q_type=q_type, q_level=q_level, device=device, weight_observer=weight_observer, quant_inference=quant_inference)
                 quant_linear.bias.data = child.bias
             else:
-                quant_linear = QuantLinear(child.in_features, child.out_features, bias=False, a_bits=a_bits, w_bits=w_bits, q_type=q_type, q_level=q_level, device=device, weight_observer=weight_observer)
+                quant_linear = QuantLinear(child.in_features, child.out_features, bias=False, a_bits=a_bits, w_bits=w_bits, q_type=q_type, q_level=q_level, device=device, weight_observer=weight_observer, quant_inference=quant_inference)
             quant_linear.weight.data = child.weight
             module._modules[name] = quant_linear
         elif isinstance(child, nn.ReLU):
@@ -558,12 +573,12 @@ def add_quant_op(module, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu',
             quant_adaptive_avg_pool = QuantAdaptiveAvgPool2d(output_size=child.output_size, a_bits=a_bits, q_type=q_type, device=device)
             module._modules[name] = quant_adaptive_avg_pool
         else:
-            add_quant_op(child, a_bits=a_bits, w_bits=w_bits, q_type=q_type, q_level=q_level, device=device, weight_observer=weight_observer, bn_fuse=bn_fuse)
+            add_quant_op(child, a_bits=a_bits, w_bits=w_bits, q_type=q_type, q_level=q_level, device=device, weight_observer=weight_observer, bn_fuse=bn_fuse, quant_inference=quant_inference)
 
-def prepare(model, inplace=False, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu', weight_observer=0, bn_fuse=0):
+def prepare(model, inplace=False, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu', weight_observer=0, bn_fuse=0, quant_inference=False):
     if not inplace:
         model = copy.deepcopy(model)
-    add_quant_op(model, a_bits=a_bits, w_bits=w_bits, q_type=q_type, q_level=q_level, device=device, weight_observer=weight_observer, bn_fuse=bn_fuse)
+    add_quant_op(model, a_bits=a_bits, w_bits=w_bits, q_type=q_type, q_level=q_level, device=device, weight_observer=weight_observer, bn_fuse=bn_fuse, quant_inference=quant_inference)
     return model
 
 '''
@@ -595,4 +610,4 @@ class QuantConcat(nn.Module):
     def forward(self, shortcut, input):
         output = torch.cat((self.activation_quantizer_1(input), self.activation_quantizer_0(shortcut)), 1)
         return output
-'''
\ No newline at end of file
+'''
